// Built with g++ 4.9.3

#include <cassert>
#include <QApplication>
#include <QMainWindow>
#include <QOpenGLWidget>
#include <QOpenGLFunctions>
#include <QDebug>
#include "renderer.h"

class MyQOpenGLWidget : public QOpenGLWidget
{
  public:
    /// Object performing the rendering operations.
    const Renderer* renderer() const { return m_renderer; }

    /** \brief Replace renderer() with `r'
     * - \post is_set: renderer()==r
     */
    void setRenderer(Renderer* r)
    {
        m_renderer = r;
        assert("post: is_set" && renderer() == r);
    }

  protected:
    /// Override from superclass.
    void initializeGL() override
    {
        if (m_renderer != nullptr) {
            m_renderer->initializeGL();
        }
    }

    /// Override from superclass.
    void resizeGL(int w, int h) override
    {
        if (m_renderer != nullptr) {
            m_renderer->resizeGL(w, h);
        }
        m_w = w;
        m_h = h;
    }

    /// Override from superclass.
    void paintGL() override
    {
        if (m_renderer != nullptr) {
            m_renderer->paintGL();
        }
    }

    Renderer* m_renderer = nullptr;
    int m_w = 0;
    int m_h = 0;
};

int main(int argc, char** argv)
{
    QApplication app(argc, argv);

    QSurfaceFormat stereoFormat = QSurfaceFormat::defaultFormat();
    stereoFormat.setStereo(true);
    qDebug() << QT_VERSION_STR
             << "QSurfaceFormat::stereo for default=" << stereoFormat.stereo();
    QSurfaceFormat::setDefaultFormat(stereoFormat);
    stereoFormat.setStereo(true);
    qDebug() << "QSurfaceFormat::stereo" << stereoFormat.stereo();

    QMainWindow window;
    MyQOpenGLWidget* widget = new MyQOpenGLWidget();
    Renderer* renderer = new Renderer;
    widget->setRenderer(renderer);
    widget->setFormat(stereoFormat);
    window.setCentralWidget(widget);
    window.setWindowTitle(QT_VERSION_STR);

    window.show();

    qDebug() << "Format contains stereo ?"
             << (widget->format().stereo() ? "TRUE" : "FALSE");

    return app.exec();
}
