#include "renderer.h"

#include <GL/gl.h>
#include <iostream>

Renderer::Renderer() : m_initialized(false)
{
}

bool Renderer::initialized() const
{
    return m_initialized;
}

void Renderer::initializeGL()
{
    m_initialized = true;

    GLboolean gbStereoEnabled = GL_FALSE;
    glGetBooleanv(GL_STEREO, &gbStereoEnabled);
    std::cout << "glGetBooleanv(GL_STEREO) returned" << std::boolalpha
              << (gbStereoEnabled == GL_TRUE) << std::endl;
}

void Renderer::resizeGL(int w, int h)
{
    glViewport(0, 0, static_cast<GLint>(w), static_cast<GLint>(h));

    m_w = w;
    m_h = h;
}

void Renderer::paintGL()
{
    std::cout << "Renderer::paintGL()" << std::endl;

    glDrawBuffer(GL_BACK_LEFT);
    drawEye(false);
    glDrawBuffer(GL_BACK_RIGHT);
    drawEye(true);
}

void Renderer::drawEye(bool right)
{
    double l = -1.0;
    double r = 1.0;
    double step = 0.01;
    if (right) {
        l += step;
        r += step;
    } else {
        l -= step;
        r -= step;
    }

    glClearColor(0.4f, 0.5f, 0.3f, 0.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glEnable(GL_DEPTH_TEST);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    // glOrtho(l,r,-1.0,1.0,0.1,10000.0);
    glFrustum(l, r, -1.0, 1.0, 0.2, 10000.0);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glColor3f(1.0f, 0.0f, 0.0f);
    glBegin(GL_TRIANGLES);
    glVertex3f(-0.5f, 0.0f, -0.5f);
    glVertex3f(0.5f, 0.0f, -0.5f);
    glVertex3f(0.0f, 1.0f, -0.5f);

    glColor3f(0.0f, 0.0f, 1.0f);
    glVertex3f(-0.1f, 0.1f, -9000.0f);
    glVertex3f(40000.0f, 0.1f, -9000.0f);
    glVertex3f(20000.0f, 20000.1f, -9000.0f);
    glEnd();
}

Renderer::~Renderer()
{
}
